/*
 * Copyright (c) 2010 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5r2.
 * 
 * SpaceNet 2.5r2 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5r2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5r2.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.component;

import java.awt.Component;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.DefaultCellEditor;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellEditor;

import edu.mit.spacenet.domain.ClassOfSupply;
import edu.mit.spacenet.domain.resource.Demand;
import edu.mit.spacenet.domain.resource.GenericResource;
import edu.mit.spacenet.domain.resource.I_Resource;
import edu.mit.spacenet.domain.resource.Item;
import edu.mit.spacenet.domain.resource.Resource;
import edu.mit.spacenet.domain.resource.ResourceType;
import edu.mit.spacenet.gui.renderer.ResourceTypeTableCellRenderer;

/**
 * A table used to display a set of demands with editable fields.
 * 
 * @author Paul Grogan
 */
public class DemandTable extends JTable {
	private static final long serialVersionUID = -5960985748957372743L;
	
	private List<I_Resource> resourceTypeList;
	private CustomTableModel model;
	
	/**
	 * The constructor.
	 * 
	 * @param list the list of available resources
	 */
	public DemandTable(List<I_Resource> list) {
		super();
		this.resourceTypeList = list;
		
		model = new CustomTableModel();
		setModel(model);
		getColumnModel().getColumn(0).setHeaderValue("Type");
		getColumnModel().getColumn(0).setCellRenderer(new ResourceTypeTableCellRenderer());
		getColumnModel().getColumn(0).setMaxWidth(100);
		getColumnModel().getColumn(1).setHeaderValue("Resource");
		getColumnModel().getColumn(2).setHeaderValue("Amount");
		getColumnModel().getColumn(2).setCellRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 2092491034324672219L;
			private DecimalFormat format = new DecimalFormat("0.00");
			public Component getTableCellRendererComponent(JTable table,
					Object value, boolean isSelected, boolean hasFocus, int row, int column) {
				
				super.getTableCellRendererComponent(table, format.format(value), isSelected, hasFocus, row, column);
				return this;
			}
		});
		getColumnModel().getColumn(2).setMaxWidth(75);
		getColumnModel().getColumn(3).setHeaderValue("Units");
		getColumnModel().getColumn(3).setMaxWidth(75);
		DefaultTableCellRenderer renderer = new DefaultTableCellRenderer();
		renderer.setHorizontalAlignment(JLabel.CENTER);
		getColumnModel().getColumn(3).setCellRenderer(renderer);
		getTableHeader().setReorderingAllowed(false);
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.JTable#getCellEditor(int, int)
	 */
	public TableCellEditor getCellEditor(int row, int col) {
    	if(col==0) {
    		JComboBox comboBox = new JComboBox();
    		comboBox.setRenderer(new DefaultListCellRenderer() {
				private static final long serialVersionUID = -8350081466318995133L;
				public Component getListCellRendererComponent(JList list, Object value, 
						int index, boolean isSelected, boolean cellHasFocus) {
					JLabel label = (JLabel)super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
					label.setIcon(((ResourceType)value).getIcon());
					return label;
				}
	    	});
    		for(ResourceType t : ResourceType.values()) {
    			comboBox.addItem(t);
    		}
    		return new DefaultCellEditor(comboBox);
    	} else if(col == 1){
	    	JComboBox comboBox = new JComboBox();
	    	for(I_Resource r : resourceTypeList) {
	    		if(this.getModel().getValueAt(row, 0).equals(ResourceType.RESOURCE)) {
	    			if(r.getClass().equals(Resource.class))
	    				comboBox.addItem(r);
	    		} else if(this.getModel().getValueAt(row, 0).equals(ResourceType.ITEM)) {
	    			if(r.getClass().equals(Item.class))
	    				comboBox.addItem(r);
	    		}
	    	}
	    	if(this.getModel().getValueAt(row, 0).equals(ResourceType.GENERIC)) {
	    		for(ClassOfSupply cos : ClassOfSupply.values()) {
	    			comboBox.addItem(new GenericResource(cos));
	            }
	    	}
	    	return new DefaultCellEditor(comboBox);
    	} else {
    		return super.getCellEditor(row, col);
    	}
    }
	private class CustomTableModel extends AbstractTableModel {
		private static final long serialVersionUID = -678300273079091701L;
		private List<Demand> data;
		private Map<Demand,ResourceType> dataType;
		
		public CustomTableModel() {
			super();
			data = new ArrayList<Demand>();
			dataType = new HashMap<Demand,ResourceType>();
		}
		public int getColumnCount() {
			return 4;
		}
		public int getRowCount() {
			return data.size();
		}
		public boolean isCellEditable(int row, int col) {
	    	if(col==3) return false;
	    	else return true;
	    }
		public Class<?> getColumnClass(int col) {
			if(col==0) {
				return ResourceType.class;
			} else if(col==1) {
				return Demand.class;
			} else if(col==2) {
				return Double.class;
			} else return String.class;
	    }
		public Object getValueAt(int row, int col) {
			if(col==0) {
				return dataType.get(data.get(row));
			} else if(col==1) {
				return data.get(row).getResource();
			} else if(col==2) {
				return data.get(row).getAmount();
			} else {
				if(data.get(row).getResource() != null)
					return data.get(row).getResource().getUnits();
				else return "";
			}
		}
	    public void setValueAt(Object value, int row, int col) {
	    	if(col==0) {
	    		dataType.put(data.get(row), (ResourceType)value);
	    	} else if(col==1) {
	    		data.get(row).setResource((I_Resource)value);
		    	fireTableCellUpdated(row, 3);
	    	} else {
	    		data.get(row).setAmount((Double)value);
	    	}
	    	fireTableCellUpdated(row, col);
	    }
		public List<Demand> getData() {
			return data;
		}
		public void addRow(Demand demand) {
			data.add(demand);
			if(demand.getResource() == null) {
				dataType.put(demand, ResourceType.GENERIC);
			} else if(demand.getResource().getResourceType()==ResourceType.GENERIC) {
				dataType.put(demand, ResourceType.GENERIC);
			} else if(demand.getResource().getResourceType()==ResourceType.RESOURCE) {
				dataType.put(demand, ResourceType.RESOURCE);
			} else {
				dataType.put(demand, ResourceType.ITEM);
			}
			fireTableRowsInserted(data.indexOf(demand), data.indexOf(demand));
		}
		public void remove(Demand demand) {
			int index = data.indexOf(demand);
			data.remove(demand);
			fireTableRowsDeleted(index, index);
		}
	}
	
	/**
	 * Adds the demand.
	 * 
	 * @param demand the demand
	 */
	public void addDemand(Demand demand) {
		model.addRow(demand);
	}
	
	/**
	 * Gets the demands.
	 * 
	 * @return the demands
	 */
	public List<Demand> getDemands() {
		return model.getData();
	}
	
	/**
	 * Gets the demand.
	 * 
	 * @param row the row
	 * 
	 * @return the demand
	 */
	public Demand getDemand(int row) {
		return model.getData().get(row);
	}
	
	/**
	 * Removes the.
	 * 
	 * @param demand the demand
	 */
	public void remove(Demand demand) {
		model.remove(demand);
	}
	
	/**
	 * Removes the all demands.
	 */
	public void removeAllDemands() {
		while (model.getRowCount()>0){
			model.remove(getDemand(0));
		}
	}
}
